// Demo 4

// A Better Hello World Java Graphics Application

// Adapted from Kernighan and Ritchie's classic C program,
//   and Deitel and Deitel's CloseWindowAndExit program.
// Adaptation by Charles W. Neville, 1998.

// All this program does is draw "Hello World" on a Frame,
//   but you can end the program gracefully by clicking on
//   the applications's window close box.

import java.awt.*;
import java.awt.event.*; // have to import Java's AWT event classes so
                         // Java will know what a WindowListener and
                         // WindowAdapter are.

// if we had defined class CloseAndExit below in a separate file, we
//   would have to import CloseAndExit here.

// Note: Applications must have their GUI components contained
//       in Frames.  Nothing will be displayed if an application
//       tries to start by displaying a Panel.
//
// No more clumsy CTRL C to end program.  This version of HelloApplication
// closes itself and exits gracefully when the Frame's close box
// is clicked.

class BetterHelloApplication extends Frame {

   // Constructor for class BetterHelloApplication
   public BetterHelloApplication() {

      // Register an instance of CloseAndExit as a
      //   window event listener so we can close the Frame and exit
      //   the app gracefully
      addWindowListener( new CloseAndExit() );

   }

   // The Java run-time system calls paint periodically to redraw the
   //    application Frame, so the paint method is a good place to draw
   //    text and images on an application Frame.
   public void paint( Graphics g ) {

     g.drawString("Hello World", 50, 100);
   }

   // Java applications are stand-alone programs.
   // Every Java application, including this one, must have a method
   //    named public static void main(String[] args).  Method main
   //    is called first whenever a stand-alone program is run.
   public static void main(String[] args) {

     BetterHelloApplication h;

     // Construct an instance of the BetterHelloApplication class
     // (a BetterHelloApplication object).
     h = new BetterHelloApplication();

     // A BetterHelloApplication object is a Frame with additional structure.
     // Set the Frame's size to 400 pixels across by 200 pixels down,
     //   and make the Frame visible (show the Frame).
     h.setSize(400, 200);
     h.setVisible(true);
   }
}

// More than one class may be defined in a single file.

// The WindowAdapter class is a built in Java class which implements interface 
// WindowListener (with empty methods), so we can have a window event handler 
// which closes a window and exits the application by extending the 
// WindowAdapter class and simply overriding the windowClosing method, without
// having to implement the other six methods of interface WindowListener.

class CloseAndExit extends WindowAdapter {

   public void windowClosing ( WindowEvent e ) {

       System.exit( 0 );  // End the application and return a "we ended
                          //   normally" signal to the operating system.
   }
}
